package cn.com.infosec.crypto.macs;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.ParametersWithIV;

class MacCFBBlockCipher {
   private byte[] IV;
   private byte[] cfbV;
   private byte[] cfbOutV;
   private int blockSize;
   private BlockCipher cipher = null;

   public MacCFBBlockCipher(BlockCipher cipher, int bitBlockSize) {
      this.cipher = cipher;
      this.blockSize = bitBlockSize / 8;
      this.IV = new byte[cipher.getBlockSize()];
      this.cfbV = new byte[cipher.getBlockSize()];
      this.cfbOutV = new byte[cipher.getBlockSize()];
   }

   public void init(CipherParameters params) {
      if (params instanceof ParametersWithIV) {
         byte[] var10000;
         byte[] var2;
         byte[] var3;
         ParametersWithIV params1;
         int var6;
         if ((var2 = (params1 = (ParametersWithIV)params).getIV()).length < (var3 = this.IV).length) {
            var10000 = var2;
            byte[] var10001 = var2;
            var6 = var3.length - var2.length;
            int var4 = var10001.length;
            System.arraycopy(var10000, 0, var3, var6, var4);
         } else {
            var10000 = var2;
            var6 = var3.length;
            System.arraycopy(var10000, 0, var3, 0, var6);
         }

         this.reset();
         this.cipher.init(true, params1.getParameters());
      } else {
         this.reset();
         this.cipher.init(true, params);
      }

   }

   public String getAlgorithmName() {
      return this.cipher.getAlgorithmName() + "/CFB" + this.blockSize * 8;
   }

   public int getBlockSize() {
      return this.blockSize;
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + var5 > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         byte[] var8 = this.cfbV;
         byte[] var6 = this.cfbOutV;
         this.cipher.processBlock(var8, 0, var6, 0);

         int var9;
         for(var5 = 0; var5 < (var9 = this.blockSize); ++var5) {
            out[outOff + var5] = (byte)(this.cfbOutV[var5] ^ in[inOff + var5]);
         }

         byte[] var10004 = in = this.cfbV;
         inOff = in.length - var9;
         System.arraycopy(var10004, var9, in, 0, inOff);
         int var7;
         System.arraycopy(out, outOff, in, (in = this.cfbV).length - (var7 = this.blockSize), var7);
         return this.blockSize;
      }
   }

   public void reset() {
      MacCFBBlockCipher var10000 = this;
      byte[] var10001 = this.IV;
      byte[] var2 = this.cfbV;
      int var1 = var10001.length;
      System.arraycopy(var10001, 0, var2, 0, var1);
      var10000.cipher.reset();
   }

   void getMacBlock(byte[] mac) {
      this.cipher.processBlock(this.cfbV, 0, mac, 0);
   }
}
